@@ -1,4 +1,5 @@
 # Copyright 2010-2011 Josh Durgin
+# Copyright 2020 Red Hat, Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,20 +18,19 @@
    (RADOS (Reliable Autonomic Distributed Object Store) Block Device)"""
 
 import contextlib
-import hashlib
 import logging
 import math
+import urllib
 
 from oslo_config import cfg
 from oslo_utils import encodeutils
 from oslo_utils import units
-from six.moves import urllib
 
 from glance_store import capabilities
 from glance_store.common import utils
 from glance_store import driver
 from glance_store import exceptions
-from glance_store.i18n import _, _LE, _LI
+from glance_store.i18n import _, _LE, _LI, _LW
 from glance_store import location
 
 try:
@@ -133,6 +133,13 @@ Related options:
 
 """),
     cfg.IntOpt('rados_connect_timeout', default=0,
+               deprecated_for_removal=True,
+               deprecated_since='Zed',
+               deprecated_reason="""
+This option has not had any effect in years. Users willing to set a timeout for
+connecting to the Ceph cluster should use 'client_mount_timeout' in Ceph's
+configuration file.
+""",
                help="""
 Timeout value for connecting to Ceph cluster.
 
@@ -149,6 +156,26 @@ Possible Values:
 Related options:
     * None
 
+"""),
+    cfg.BoolOpt('rbd_thin_provisioning',
+                default=False,
+                help="""
+Enable or not thin provisioning in this backend.
+
+This configuration option enable the feature of not really write null byte
+sequences on the RBD backend, the holes who can appear will automatically
+be interpreted by Ceph as null bytes, and do not really consume your storage.
+Enabling this feature will also speed up image upload and save network traffic
+in addition to save space in the backend, as null bytes sequences are not
+sent over the network.
+
+Possible Values:
+    * True
+    * False
+
+Related options:
+    * None
+
 """),
 ]
 
@@ -268,7 +295,7 @@ class Store(driver.Store):
         client = rados.Rados(conffile=conffile, rados_id=rados_id)
 
         try:
-            client.connect(timeout=self.connect_timeout)
+            client.connect()
         except (rados.Error, rados.ObjectNotFound) as e:
             if self.backend_group and len(self.conf.enabled_backends) > 1:
                 reason = _("Error in store configuration: %s") % e
@@ -299,15 +326,18 @@ class Store(driver.Store):
                 user = getattr(self.conf, self.backend_group).rbd_store_user
                 conf_file = getattr(self.conf,
                                     self.backend_group).rbd_store_ceph_conf
-                connect_timeout = getattr(
-                    self.conf, self.backend_group).rados_connect_timeout
+                thin_provisioning = getattr(self.conf,
+                                            self.backend_group).\
+                    rbd_thin_provisioning
             else:
                 chunk = self.conf.glance_store.rbd_store_chunk_size
                 pool = self.conf.glance_store.rbd_store_pool
                 user = self.conf.glance_store.rbd_store_user
                 conf_file = self.conf.glance_store.rbd_store_ceph_conf
-                connect_timeout = self.conf.glance_store.rados_connect_timeout
+                thin_provisioning = \
+                    self.conf.glance_store.rbd_thin_provisioning
 
+            self.thin_provisioning = thin_provisioning
             self.chunk_size = chunk * units.Mi
             self.READ_CHUNKSIZE = self.chunk_size
             self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
@@ -317,7 +347,6 @@ class Store(driver.Store):
             self.pool = str(pool)
             self.user = str(user)
             self.conf_file = str(conf_file)
-            self.connect_timeout = connect_timeout
         except cfg.ConfigFileValueError as e:
             reason = _("Error in store configuration: %s") % e
             LOG.error(reason)
@@ -325,6 +354,8 @@ class Store(driver.Store):
                                                    reason=reason)
         if self.backend_group:
             self._set_url_prefix()
+        self.size = 0
+        self.resize_amount = self.WRITE_CHUNKSIZE
 
     def _set_url_prefix(self):
         fsid = None
@@ -406,6 +437,14 @@ class Store(driver.Store):
             'snapshot': DEFAULT_SNAPNAME,
         }, self.conf)
 
+    def _snapshot_has_external_reference(self, image, snapshot_name):
+        """Returns True if snapshot has external reference else False.
+        """
+        image.set_snap(snapshot_name)
+        has_references = bool(image.list_children())
+        image.set_snap(None)
+        return has_references
+
     def _delete_image(self, target_pool, image_name,
                       snapshot_name=None, context=None):
         """
@@ -425,6 +464,14 @@ class Store(driver.Store):
                     if snapshot_name is not None:
                         with rbd.Image(ioctx, image_name) as image:
                             try:
+                                # NOTE(abhishekk): Check whether snapshot
+                                # has any external references
+                                if self._snapshot_has_external_reference(
+                                        image, snapshot_name):
+                                    raise rbd.ImageBusy(
+                                        "Image snapshot has external "
+                                        "references.")
+
                                 self._unprotect_snapshot(image, snapshot_name)
                                 image.remove_snap(snapshot_name)
                             except rbd.ImageNotFound as exc:
@@ -434,26 +481,26 @@ class Store(driver.Store):
                                        {'snap_exc': exc})
                                 LOG.debug(msg)
                             except rbd.ImageBusy as exc:
-                                log_msg = (_LE("Snap Operating Exception "
+                                log_msg = (_LW("Snap Operating Exception "
                                                "%(snap_exc)s "
                                                "Snapshot is in use.") %
                                            {'snap_exc': exc})
-                                LOG.error(log_msg)
+                                LOG.warning(log_msg)
                                 raise exceptions.InUseByStore()
 
                     # Then delete image.
                     rbd.RBD().remove(ioctx, image_name)
                 except rbd.ImageHasSnapshots:
-                    log_msg = (_LE("Remove image %(img_name)s failed. "
+                    log_msg = (_LW("Remove image %(img_name)s failed. "
                                    "It has snapshot(s) left.") %
                                {'img_name': image_name})
-                    LOG.error(log_msg)
+                    LOG.warning(log_msg)
                     raise exceptions.HasSnapshot()
                 except rbd.ImageBusy:
-                    log_msg = (_LE("Remove image %(img_name)s failed. "
+                    log_msg = (_LW("Remove image %(img_name)s failed. "
                                    "It is in use.") %
                                {'img_name': image_name})
-                    LOG.error(log_msg)
+                    LOG.warning(log_msg)
                     raise exceptions.InUseByStore()
                 except rbd.ImageNotFound:
                     msg = _("RBD image %s does not exist") % image_name
@@ -468,6 +515,18 @@ class Store(driver.Store):
             # Such exception is not dangerous for us so it will be just logged
             LOG.debug("Snapshot %s is unprotected already" % snap_name)
 
+    def _resize_on_write(self, image, image_size, bytes_written, chunk_length):
+        """Handle the rbd resize when needed."""
+        if image_size != 0 or self.size >= bytes_written + chunk_length:
+            return self.size
+        new_size = self.size + self.resize_amount
+        LOG.debug("resizing image to %s KiB" % (new_size / units.Ki))
+        image.resize(new_size)
+        # Note(jokke): We double how much we grow the image each time
+        # up to 8gigs to avoid resizing for each write on bigger images
+        self.resize_amount = min(self.resize_amount * 2, 8 * units.Gi)
+        return new_size
+
     @driver.back_compat_add
     @capabilities.check
     def add(self, image_id, image_file, image_size, hashing_algo, context=None,
@@ -490,8 +549,8 @@ class Store(driver.Store):
         :raises: `glance_store.exceptions.Duplicate` if the image already
                  exists
         """
-        checksum = hashlib.md5()
-        os_hash_value = hashlib.new(str(hashing_algo))
+        os_hash_value = utils.get_hasher(hashing_algo, False)
+        checksum = utils.get_hasher('md5', False)
         image_name = str(image_id)
         with self.get_connection(conffile=self.conf_file,
                                  rados_id=self.user) as conn:
@@ -504,7 +563,7 @@ class Store(driver.Store):
                 # Fix with encodeutils.safe_decode CAN BE REMOVED
                 # after librados's fix will be stable.
                 #
-                # More informations:
+                # More information:
                 # https://bugs.launchpad.net/glance-store/+bug/1816721
                 # https://bugs.launchpad.net/cinder/+bug/1816468
                 # https://tracker.ceph.com/issues/38381
@@ -514,9 +573,9 @@ class Store(driver.Store):
                 LOG.debug('creating image %s with order %d and size %d',
                           image_name, order, image_size)
                 if image_size == 0:
-                    LOG.warning(_("since image size is zero we will be doing "
-                                  "resize-before-write for each chunk which "
-                                  "will be considerably slower than normal"))
+                    LOG.warning(_LW("Since image size is zero we will be "
+                                    "doing resize-before-write which will be "
+                                    "slower than normal"))
 
                 try:
                     loc = self._create_image(fsid, conn, ioctx, image_name,
@@ -532,24 +591,27 @@ class Store(driver.Store):
                         chunks = utils.chunkreadable(image_file,
                                                      self.WRITE_CHUNKSIZE)
                         for chunk in chunks:
-                            # If the image size provided is zero we need to do
-                            # a resize for the amount we are writing. This will
-                            # be slower so setting a higher chunk size may
-                            # speed things up a bit.
-                            if image_size == 0:
-                                chunk_length = len(chunk)
-                                length = offset + chunk_length
-                                bytes_written += chunk_length
-                                LOG.debug(_("resizing image to %s KiB") %
-                                          (length / units.Ki))
-                                image.resize(length)
-                            LOG.debug(_("writing chunk at offset %s") %
-                                      (offset))
-                            offset += image.write(chunk, offset)
+                            # NOTE(jokke): If we don't know image size we need
+                            # to resize it on write. The resize amount will
+                            # ramp up to 8 gigs.
+                            chunk_length = len(chunk)
+                            self.size = self._resize_on_write(image,
+                                                              image_size,
+                                                              bytes_written,
+                                                              chunk_length)
+                            bytes_written += chunk_length
+                            if not (self.thin_provisioning and not any(chunk)):
+                                image.write(chunk, offset)
+                            offset += chunk_length
                             os_hash_value.update(chunk)
                             checksum.update(chunk)
                             if verifier:
                                 verifier.update(chunk)
+
+                        # Lets trim the image in case we overshoot with resize
+                        if image_size == 0:
+                            image.resize(bytes_written)
+
                         if loc.snapshot:
                             image.create_snap(loc.snapshot)
                             image.protect_snap(loc.snapshot)
@@ -592,7 +654,7 @@ class Store(driver.Store):
         # Add store backend information to location metadata
         metadata = {}
         if self.backend_group:
-            metadata['store'] = u"%s" % self.backend_group
+            metadata['store'] = self.backend_group
 
         return (loc.get_uri(),
                 image_size,
