@@ -13,10 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+import io
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_utils import units
-import six
 
 import glance_store as store
 from glance_store._drivers import rbd as rbd_store
@@ -112,6 +113,12 @@ class MockRBD(object):
         def remove_snap(self, *args, **kwargs):
             pass
 
+        def set_snap(self, *args, **kwargs):
+            pass
+
+        def list_children(self, *args, **kwargs):
+            pass
+
         def protect_snap(self, *args, **kwargs):
             pass
 
@@ -215,7 +222,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                                                 self.conf)
         # Provide enough data to get more than one chunk iteration.
         self.data_len = 3 * units.Ki
-        self.data_iter = six.BytesIO(b'*' * self.data_len)
+        self.data_iter = io.BytesIO(b'*' * self.data_len)
 
     def test_location_url_prefix_is_set(self):
         expected_url_prefix = "rbd://"
@@ -246,7 +253,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                                                 self.conf)
         # Provide enough data to get more than one chunk iteration.
         self.data_len = 3 * units.Ki
-        self.data_iter = six.BytesIO(b'*' * self.data_len)
+        self.data_iter = io.BytesIO(b'*' * self.data_len)
         self.store.chunk_size = units.Ki
         with mock.patch.object(rbd_store.rbd.Image, 'resize') as resize:
             with mock.patch.object(rbd_store.rbd.Image, 'write') as write:
@@ -416,6 +423,15 @@ class TestMultiStore(base.MultiStoreBaseTest,
 
             self.called_commands_expected = ['unprotect_snap']
 
+    def test_delete_image_snap_has_external_references(self):
+        with mock.patch.object(MockRBD.Image, 'list_children') as mocked:
+            mocked.return_value = True
+
+            self.assertRaises(exceptions.InUseByStore,
+                              self.store._delete_image,
+                              'fake_pool', self.location.image,
+                              snapshot_name='snap')
+
     def test_delete_image_w_snap_exc_image_has_snap(self):
         def _fake_remove(*args, **kwargs):
             self.called_commands_actual.append('remove')
@@ -434,14 +450,6 @@ class TestMultiStore(base.MultiStoreBaseTest,
         self.assertRaises(exceptions.StoreRandomGetNotSupported,
                           self.store.get, loc, chunk_size=1)
 
-    @mock.patch.object(MockRados.Rados, 'connect')
-    def test_rados_connect_timeout(self, mock_rados_connect):
-        socket_timeout = 1
-        self.config(rados_connect_timeout=socket_timeout, group="ceph1")
-        self.store.configure()
-        with self.store.get_connection('conffile', 'rados_id'):
-            mock_rados_connect.assert_called_with(timeout=socket_timeout)
-
     @mock.patch.object(MockRados.Rados, 'connect', side_effect=MockRados.Error)
     def test_rados_connect_error(self, _):
         rbd_store.rados.Error = MockRados.Error
